<?php declare(strict_types=1);

namespace FeedReaderCentral;

use BitBadger\PDODocument\{Configuration, Custom, DocumentException, Field, Parameters, Query};
use BitBadger\PDODocument\Mapper\{DocumentMapper, ExistsMapper};

/**
 * A combined item and feed (used for lists)
 */
class ItemWithFeed extends Item
{
    /** @var string The body of the `FROM` clause to join item and feed */
    public const FROM_WITH_JOIN = Table::ITEM . ' INNER JOIN ' . Table::FEED
        . ' ON ' . Table::ITEM . ".data->>'feed_id' = " . Table::FEED . ".data->>'id'";

    /** @var string The `SELECT` clause to add the feed as a property to the item's document */
    public const SELECT_WITH_FEED =
        'SELECT json_set(' . Table::ITEM . ".data, '$.feed', json(" . Table::FEED . '.data)) AS data FROM '
        . self::FROM_WITH_JOIN;

    /** @var Feed The feed to which this item belongs */
    public Feed $feed;

    /**
     * Create JSON comparison fields to retrieve items while also checking the owning user
     *
     * @param int $id The ID of the item being retrieved
     * @return array|Field[] The fields for item ID and user ID
     */
    private static function idAndUserFields(int $id): array
    {
        $idField = Field::EQ(Configuration::$idField, $id, ':id');
        $idField->qualifier = Table::ITEM;
        $userField = Field::EQ('user_id', $_SESSION[Key::USER_ID], ':user');
        $userField->qualifier = Table::FEED;
        return [$idField, $userField];
    }

    /**
     * Check an item's existence via its ID
     *
     * @param int $id The ID of the item whose existence should be checked
     * @return bool True if the item exists for the current user, false if not
     * @throws DocumentException If any is encountered
     */
    public static function existsById(int $id): bool
    {
        $fields = self::idAndUserFields($id);
        return Custom::scalar(Query\Exists::query(self::FROM_WITH_JOIN, Query::whereByFields($fields)),
            Parameters::addFields($fields, []), new ExistsMapper());
    }

    /**
     * Retrieve an item via its ID
     *
     * @param int $id The ID of the item to be retrieved
     * @return ItemWithFeed|false The item if it is found, false if not
     * @throws DocumentException If any is encountered
     */
    public static function retrieveById(int $id): ItemWithFeed|false
    {
        $fields = self::idAndUserFields($id);
        return Custom::single(self::SELECT_WITH_FEED . ' WHERE ' . Query::whereByFields($fields),
            Parameters::addFields($fields, []), new DocumentMapper(self::class));
    }
}
